package com.idea.search.service;

import com.alibaba.fastjson.JSON;
import com.idea.search.dao.EsOpLogMapper;
import com.idea.search.pojo.OpLogRecord;
import com.idea.search.pojo.UserRecord;
import com.idea.search.util.IndexTypeEnum;
import com.rockstar.frame.model.FrameOpLog;
import com.rockstar.util.BeanUtils;
import com.rockstar.util.DateUtils;
import org.apache.commons.lang.StringUtils;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.Operator;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.SearchResultMapper;
import org.springframework.data.elasticsearch.core.aggregation.AggregatedPage;
import org.springframework.data.elasticsearch.core.aggregation.impl.AggregatedPageImpl;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * ES服务器搭建：https://blog.csdn.net/alexyangxb/article/details/115898520
 */
@Service
public class EsService {

    //es模板类
    @Autowired
    private ElasticsearchTemplate esTemplate;

    @Autowired
    private EsOpLogMapper opLogMapper;

    //创建索引库结构
    public void createIndexAndMapping(String indexCode) {
        IndexTypeEnum typeEnum = IndexTypeEnum.getByCode(indexCode);
        switch (typeEnum){
            case OP_LOG:
                esTemplate.createIndex(OpLogRecord.class); //创建索引(根据索引库映射实体类)
                esTemplate.putMapping(OpLogRecord.class); //创建映射
            case FRAME_USER:
                esTemplate.createIndex(UserRecord.class); //创建索引(根据索引库映射实体类)
                esTemplate.putMapping(UserRecord.class); //创建映射
        }
    }

    public OpLogRecord saveOpLog(FrameOpLog opLog){
        OpLogRecord esLog = new OpLogRecord();
        BeanUtils.copyBeanProp(esLog, opLog);
        esLog.setOperAt(DateUtils.formatDateTime(opLog.getOperAt()));
        return opLogMapper.save(esLog);
    }

    public Object opLogQuery(Map<String, String> searchMap) {
        NativeSearchQueryBuilder queryBuilder = genQueryBuilder(searchMap, 1500, true, "operAt");
        AggregatedPage<OpLogRecord> fr = esTemplate.queryForPage(queryBuilder.build(), OpLogRecord.class, new PageResultMapper());
        return fr.getContent();
    }

    private NativeSearchQueryBuilder genQueryBuilder(Map<String, String> searchMap, Integer totalSize, boolean isDate, String orderBy) {
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();//可以通过boolQuery拼接查询条件

        if (!searchMap.isEmpty()) {
            searchMap.forEach((key, value) -> {
                if (key.equals("title")) {
                    boolQuery.must(QueryBuilders.matchQuery("title", value).operator(Operator.AND));
                } else if (key.equals("start")) {
                    if (isDate) {
                        value = value + " 00:00:00";
                    }
                    boolQuery.filter(QueryBuilders.rangeQuery("operAt").gte(value));
                } else if (key.equals("end")) {
                    if (isDate) {
                        value = value + " 23:59:59";
                    }
                    boolQuery.filter(QueryBuilders.rangeQuery("operAt").lte(value));
                } else {
                    //过滤条件, filter()方法  termQuery() 完全条件匹配查询(精确查询)
                    boolQuery.filter(QueryBuilders.termQuery(key, value));
                }
            });
        }

        NativeSearchQueryBuilder queryBuilder = new NativeSearchQueryBuilder();
        queryBuilder.withQuery(boolQuery);//查询方法

        if (null != totalSize) {//分页
            queryBuilder.withPageable(PageRequest.of(0, totalSize));
        }
        if (StringUtils.isNotEmpty(orderBy)) {//排序
            queryBuilder.withSort(SortBuilders.fieldSort(orderBy).order(SortOrder.ASC));
        }

        return queryBuilder;
    }

    class PageResultMapper implements SearchResultMapper {

        public PageResultMapper() { }

        @Override
        public <T> AggregatedPage<T> mapResults(SearchResponse searchResponse, Class<T> aClass, Pageable pageable) {
            //查询结果的相关操作 将数据查询结果的 转换为实体类
            List<T> list = new ArrayList<>();
            //获取查询命中结果数据
            SearchHits hits = searchResponse.getHits();
            if (hits != null) {
                //不为空 有查询结果
                for (SearchHit hit : hits) {
                    //当前得到的对象为 得到的每一条记录
                    //得到的 SearchHits对象转换为 实体类
                    //用阿里json转换
                    OpLogRecord fr = JSON.parseObject(hit.getSourceAsString(), OpLogRecord.class);
                    list.add((T) fr);
                }
            }
            /**
             * 集合 ,分页信息,总记录数,构造
             */
            return new AggregatedPageImpl<T>(list, pageable, hits.getTotalHits(), searchResponse.getAggregations());
        }

        @Override
        public <T> T mapSearchHit(SearchHit searchHit, Class<T> aClass) {
            return null;
        }
    }
}
